'''User Interface with documentation '''

from ugd.high_level_interface.draw_simulations import hyp_test


def digraph_hyp_test(adj_m, var_dict=None, stat_f=None, test_variable=None, controls=None, mixing_time=None,
                     anz_sim=1000, show_plot=False, fast_mixing_time_evaluation = False):
    
    
    '''
    
    
    OVERVIEW: This function generates random digraphs with given in- and out- degree sequences 
              and "cross-arrow" (or cross-link) restrictions. Digraphs are drawn uniformly at 
              random from the set of all labeled graphs obeying the given constraints. The simulated
              digraphs are used to estimate the H0/Null distribution of a test statistic. Further 
              explanation of methods is given in the paper : “An optimal test for strategic 
              interaction in social and economic network formation between heterogeneous agents” by
              Andrin Pelican and Bryan Graham (see https://arxiv.org/abs/2009.00212).

    
    INPUT:
    -----------------------------------------------------------------------------------------------------    
    :param adj_m:         A numpy array containing 0 and 1s as elements, representing adjacency 
                          matrix of the digraph.
    
    :param var_dict:      A dictionary with the integer 1..n as primary Key, where n is the 
                          number of nodes, the Values are also dictionaries (with the variable 
                          names as Keys and variable values as Values). Values have to be numbers 
                          or strings.
    
    :param stat_f:        A function which maps the adj_m and var_dict to a real number 
                          ("The statistic of interest").
    
    :param test_variable: Alternative to stat_f, creating a statistic which counts arcs from 
                          one node subset into another: a triple with first element equal to a 
                          variable name, second element equal to the value of the variable 
                          for the node set where the arcs are directed from, and third element 
                          the value for the subset where the arcs are directed to.
    
    :param controls:      List of variable names for the controls used to define the cross-link 
                          matrix.
    
    :param mixing_time:   Number of runs (steps in the markov graph) before a the graph is 
                          considered a unifom random draw from the target set.
    
    :param anz_sim:       Number graphs to simulate/generate.
    
    :param show_plot:     Boolean for whether a plot is desired.
    
    
    OUTPUT:
    -----------------------------------------------------------------------------------------------------        
    graph_list:           List of random adjacency matrices with the given in- and out- degree sequences 
                          and "cross-arrow" (or cross-link) restrictions.
    
    stats_list:           List of the statistics, stat_f, evaluated for the simulated graphs.

    
    USAGE:
    -----------------------------------------------------------------------------------------------------        
    import numpy
    adj_m = numpy.zeros((4,4))
    adj_m[0,1] = 1
    adj_m[2,3] = 1
    var_dict ={
        0: {'gender': 'm'},
        1: {'gender': 'm'},
        2: {'gender': 'f'},
        3: {'gender': 'f'},
    }
    graphs, stats_list = digraph_hyp_test(adj_m = adj_m, var_dict = var_dict, 
                                          test_variable = ('gender','m','f'), 
                                          mixing_time = 10000, anz_sim = 100, show_plot=True)

    AUTHORS:
    Andrin Pelican, andrin.pelican@bluewin.ch

    '''

    return hyp_test(adj_m, var_dict, stat_f, test_variable, controls, mixing_time, anz_sim, show_plot,
                    is_directed=True,  fast_mixing_time_evaluation = fast_mixing_time_evaluation)


def graph_hyp_test(adj_m, var_dict=None, stat_f=None, test_variable=None, controls=None, mixing_time=None,
                   anz_sim=1000, show_plot=False,  fast_mixing_time_evaluation = False):
    '''
    
    
    OVERVIEW: generation of random (undirected) Graphs with a given degree sequence and 
              restrictions induced by node attributes in order to estimate the distribution 
              of a test-statistic under the H0/null Hypothesis. Further explanation is
              given in the paper: "Uniform Sampling of Graphs with Fixed Degree Sequence 
              under Partition Constraints" by Andrin Pelican.

    
    INPUT:
    -----------------------------------------------------------------------------------------------------        
    :param adj_m:         A numpy array containing 0 and 1s as elements, representing 
                          adjacency matrix of the graph
    
    :param var_dict:      A dictionary with the integers 1..n as primary Key (representing 
                          the n nodes).
                          The Values are dictionaries containing the Variable name as Keys 
                          and Values which can either be numbers or strings.
    
    :param stat_f:        A function which maps the adj_m and var_dict to a number 
                          ("The statistic of interest").
    
    :param test_variable: Alternative to stat_f, creating a statistic which counts the edges 
                          from one node-subset into another: a triple with first element equal 
                          to a variable name, second element equal to the value of the variable 
                          for the node set where the edges are from, and third element the value 
                          for the subset where the edges go.
    
    :param controls:      List of variable names for the controls used to define the cross-link 
                          matrix.
    
    :param mixing_time:   Number of runs (steps in the markov graph) before a the graph is 
                          considered a unifom random draw from the target set.
    
    :param anz_sim:       Number graphs to simulate/generate.
    
    :param show_plot:     Boolean whether a plot is desired.

    
    OUTPUT:
    -----------------------------------------------------------------------------------------------------    
    graph_list:           List of random adjacency matrices with the given degree sequence 
                          cross-link restrictions.
    
    stats_list:           List of the statistics, stat_f, evaluated for the simulated graphs.

    
    USAGE:
    -----------------------------------------------------------------------------------------------------        
    import numpy
    adj_m = numpy.zeros((4,4))
    adj_m[0,1] = 1
    adj_m[1,0] = 1
    adj_m[3,2] = 1
    adj_m[2,3] = 1
    var_dict ={
        0: {'gender': 'm'},
        1: {'gender': 'm'},
        2: {'gender': 'f'},
        3: {'gender': 'f'},
    }
    graphs, stats_list = graph_hyp_test(adj_m = adj_m, var_dict = var_dict, 
                                        test_variable= ('gender','m','f'), 
                                        mixing_time = 10000, anz_sim = 100, show_plot=True)

    AUTHORS:
    Andrin Pelican, andrin.pelican@bluewin.ch

    '''

    return hyp_test(adj_m, var_dict, stat_f, test_variable, controls, mixing_time, anz_sim, show_plot,
                    is_directed=False,  fast_mixing_time_evaluation = fast_mixing_time_evaluation)



from ugd import digraph_hyp_test
from ugd import graph_hyp_test

import numpy as np
adj_m = np.zeros((4,4))
adj_m[0,1] = 1
adj_m[2,3] = 1


ouput_dict = digraph_hyp_test(adj_m=adj_m)

print(ouput_dict["graph_list"])




import numpy as np
import unittest
import ugd
from test.test_resources.plain_graph_integration import adj_m_1, var_dict_1, adj_m_2, var_dict_2

class TestIntegrationGraph(unittest.TestCase):

    def test_graph_no_rstrc(self):
        output_dict= ugd.graph_hyp_test(adj_m=adj_m_1, var_dict=var_dict_1, test_variable=('gender', 'm', 'f'), mixing_time=100, anz_sim=1000, show_plot=False)
        mue = np.mean(output_dict['stat_list'])
        true_mean = 0*1/3+ 2/3 * 2 # in this simple case the true mean can be calculated manually

        true_normalized_numb_graphs_same_as_observed = 1/3
        true_quantil_with_50_50_rule = 1/6

        assert output_dict["info_dict"]["normalized_numb_graphs_same_as_observed"] > true_normalized_numb_graphs_same_as_observed - 0.15 and output_dict["info_dict"]["normalized_numb_graphs_same_as_observed"] < true_normalized_numb_graphs_same_as_observed + 0.15
        assert output_dict["info_dict"]["quantile"] > true_quantil_with_50_50_rule - 0.15 and output_dict["info_dict"]["quantile"] < true_quantil_with_50_50_rule + 0.15
        assert mue > true_mean - 0.15 and mue < true_mean + 0.15




import matplotlib.pyplot as plt
import numpy as np
from ugd.help_function.util_statistic import get_quantile, get_normalized_numb_graphs_same_as_observed


def postprocess(adj_m_original, stats_list, stat_f, var_dict, test_variable, edges_changed_per_draw, show_plot=False):
    '''
    General output postprocessing and plotting, this part is convenient for experimenting with graph.
    However it can easily be customized by using the output graph sequence of the algorithm.
    '''
    org_value = stat_f(adj_m_original, var_dict)
    quantile = get_quantile(org_value, stats_list)
    normalized_numb_graphs_same_as_observed = get_normalized_numb_graphs_same_as_observed(org_value, stats_list)


    statistic_name = stat_f.__name__
    if not (test_variable == None):
        statistic_name = 'crosslinks ' + str(test_variable[0]) + ': ' + str(test_variable[1]) + ' to ' + str(test_variable[2])

    info_dict = {
        'stat_name': statistic_name,
        'original_value': org_value,
        'normalized_numb_graphs_same_as_observed': normalized_numb_graphs_same_as_observed,
        'quantile': quantile,
    }

    annotate_string = 'Estimated H0 distribution for statistic: ' + statistic_name               + '\n' \
                    + 'Observed statistic lies at quantile: ' + '{:.4f}'.format(quantile)    + '\n' \
                    + '(of the simulated H0 distribution)'                                            + '\n' \
                    + 'Average numbers of edges changed per draw: ' + str(int(edges_changed_per_draw * 100)) + '%'

    
    #min_value = min(stats_list)
    #max_value = max(stats_list)
    #y = [0] * (int(max_value + 1) - int(min_value))
    #x = list(range(int(min_value), int(max_value + 1)))
    #for stat_val in stats_list:
    #    y[int(stat_val) - int(min_value)] += 1 / stats_list.__len__()

    #width = 1 / 1.5
    #plt.bar(x, y, width, color= "#3B7EA1")

    plt.hist(stats_list, bins='fd', density = True)
    plt.axvline(x=org_value, color="#FDB515")
    plt.title('H0 Reference Distribution')
    plt.ylabel('Frequency')
    plt.xlabel('Statistic')
    plt.gcf().subplots_adjust(bottom=0.25)
    plt.annotate(annotate_string, (0, 0), (0, -35), xycoords='axes fraction', textcoords='offset points', va='top')

    if show_plot:
        plt.show()
    else:
        plt.clf()
        plt.cla()
        plt.close()

    return plt, info_dict
